@@ -25,7 +25,7 @@
 ### the mle function of the stat package.
 
 mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="default", 
-    lower=-Inf, upper=Inf, custom.optim=NULL, weights=NULL, silent=TRUE, ...)
+    lower=-Inf, upper=Inf, custom.optim=NULL, weights=NULL, silent=TRUE, gradient=NULL, ...)
     # data may correspond to a vector for non censored data or to
     # a dataframe of two columns named left and right for censored data 
 {
@@ -37,6 +37,8 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
     
     if (!exists(ddistname, mode="function"))
         stop(paste("The ", ddistname, " function must be defined"))
+    if(is.null(custom.optim))
+      optim.method <- match.arg(optim.method, c("default", "Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN", "Brent"))
 
     start.arg <- start #to avoid confusion with the start() function of stats pkg (check is done lines 87-100)
     if(is.vector(start.arg)) #backward compatibility
@@ -47,9 +49,12 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
     if(!is.null(weights))
     {
       if(any(weights < 0))
-        stop("weights should be a vector of numerics greater than 0")
+        stop("weights should be a vector of integers greater than 0")
+      if(!is.allint.w(weights))
+        stop("weights should be a vector of (strictly) positive integers")
       if(length(weights) != NROW(data))
         stop("weights should be a vector with a length equal to the observation number")
+      warning("weights are not taken into account in the default initial values")
     }
     
     if (is.vector(data)) {
@@ -72,14 +77,18 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
         # Definition of datasets lcens (left censored)=vector, rcens (right censored)= vector, 
         #   icens (interval censored) = dataframe with left and right 
         # and ncens (not censored) = vector
-        lcens<-censdata[is.na(censdata$left), ]$right
+        irow.lcens <- is.na(censdata$left) # rows corresponding to lcens data
+        lcens <- censdata[irow.lcens, ]$right
         if (any(is.na(lcens)) )
             stop("An observation cannot be both right and left censored, coded with two NA values")
-        rcens<-censdata[is.na(censdata$right), ]$left
-        ncens<-censdata[censdata$left==censdata$right & !is.na(censdata$left) & 
-            !is.na(censdata$right), ]$left
-        icens<-censdata[censdata$left!=censdata$right & !is.na(censdata$left) & 
-            !is.na(censdata$right), ]
+        irow.rcens <- is.na(censdata$right)  # rows corresponding to rcens data
+        rcens <- censdata[irow.rcens, ]$left
+        irow.ncens <- censdata$left==censdata$right & !is.na(censdata$left) & 
+                      !is.na(censdata$right)  # rows corresponding to ncens data
+        ncens<-censdata[irow.ncens, ]$left
+        irow.icens <- censdata$left!=censdata$right & !is.na(censdata$left) & 
+          !is.na(censdata$right)  # rows corresponding to icens data
+        icens<-censdata[irow.icens, ]
         # Definition of a data set for calculation of starting values
         data<-c(rcens, lcens, ncens, (icens$left+icens$right)/2)
     }
@@ -92,7 +101,7 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
       stop(chfixstt$txt)
     #unlist starting values as needed in optim()
     if(is.function(chfixstt$start.arg))
-      vstart <- chfixstt$start.arg(data)
+      vstart <- unlist(chfixstt$start.arg(data))
     else
       vstart <- unlist(chfixstt$start.arg)
     if(is.function(fix.arg)) #function
@@ -167,40 +176,126 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
         p3 <- log(1-do.call(pdistnam, c(list(q=rcens), as.list(par), as.list(fix.arg))))
         p4 <- log(do.call(pdistnam, c(list(q=icens$right), as.list(par), as.list(fix.arg))) - 
                     do.call(pdistnam, c(list(q=icens$left), as.list(par), as.list(fix.arg))) )
-        -sum(weights * p1) - sum(weights * p2) - sum(weights * p3) - sum(weights * p4) 
+        -sum(weights[irow.ncens] * p1) - 
+          sum(weights[irow.lcens] * p2) - 
+          sum(weights[irow.rcens] * p3) - 
+          sum(weights[irow.icens] * p4) 
       }
-    }else
-        stop("not yet implemented.")
+    }
    
-    # Choice of the optimization method    
-    if (optim.method == "default")
-    {
-        if(is.infinite(lower) && is.infinite(upper))
-        { 
-            if (length(vstart) > 1) 
-                meth <- "Nelder-Mead"
-            else 
-                meth <- "BFGS"
-        }else
-            meth <- "L-BFGS-B"
-    }else
-        meth <- optim.method
-        
+    #get warning value    
     owarn <- getOption("warn")
     
     # Try to minimize the minus (log-)likelihood using the base R optim function
     if(is.null(custom.optim))
     {
+        hasbound <- any(is.finite(lower) | is.finite(upper))
+        
+        # Choice of the optimization method  
+        if (optim.method == "default")
+        {
+          meth <- ifelse(length(vstart) > 1, "Nelder-Mead", "BFGS") 
+        }else
+          meth <- optim.method
+        
+        if(meth == "BFGS" && hasbound && is.null(gradient))
+        {
+          meth <- "L-BFGS-B"
+          txt1 <- "The BFGS method cannot be used with bounds without provided the gradient."
+          txt2 <- "The method is changed to L-BFGS-B."
+          warning(paste(txt1, txt2))
+        }
+        
         options(warn=ifelse(silent, -1, 0))
-        if (!cens)
-            opttryerror <- try(opt <- optim(par=vstart, fn=fnobj, fix.arg=fix.arg, obs=data, 
-                ddistnam=ddistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
-                ...), silent=TRUE)        
-        else 
-            opttryerror <- try(opt <- optim(par=vstart, fn=fnobjcens, fix.arg=fix.arg, 
-                rcens=rcens, lcens=lcens, icens=icens, ncens=ncens, ddistnam=ddistname, 
-                pdistnam=pdistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
-                ...), silent=TRUE)   
+        #select optim or constrOptim
+        if(hasbound) #finite bounds are provided
+        {
+          if(!is.null(gradient))
+          {
+            opt.fun <- "constrOptim"
+          }else #gradient == NULL
+          {
+            if(meth == "Nelder-Mead")
+              opt.fun <- "constrOptim"
+            else if(meth %in% c("L-BFGS-B", "Brent"))
+              opt.fun <- "optim"
+            else
+            {
+              txt1 <- paste("The method", meth, "cannot be used by constrOptim() nor optim() without gradient and bounds.")
+              txt2 <- "Only optimization methods L-BFGS-B, Brent and Nelder-Mead can be used in such case."
+              stop(paste(txt1, txt2))
+            }
+          }
+          if(opt.fun == "constrOptim")
+          {
+            #recycle parameters
+            npar <- length(vstart) #as in optim() line 34
+            lower <- as.double(rep_len(lower, npar)) #as in optim() line 64
+            upper <- as.double(rep_len(upper, npar))
+            
+            # constraints are : Mat %*% theta >= Bnd, i.e. 
+            # +1 * theta[i] >= lower[i]; 
+            # -1 * theta[i] >= -upper[i]
+            
+            #select rows from the identity matrix
+            haslow <- is.finite(lower)
+            Mat <- diag(npar)[haslow, ]
+            #select rows from the opposite of the identity matrix
+            hasupp <- is.finite(upper)
+            Mat <- rbind(Mat, -diag(npar)[hasupp, ])
+            colnames(Mat) <- names(vstart)
+            rownames(Mat) <- paste0("constr", 1:NROW(Mat))
+            
+            #select the bounds
+            Bnd <- c(lower[is.finite(lower)], -upper[is.finite(upper)])
+            names(Bnd) <- paste0("constr", 1:length(Bnd))
+            
+            initconstr <- Mat %*% vstart - Bnd
+            if(any(initconstr < 0))
+              stop("Starting values must be in the feasible region.")
+            
+            if(!cens)
+            {
+              opttryerror <- try(opt <- constrOptim(theta=vstart, f=fnobj, ui=Mat, ci=Bnd, grad=gradient,
+                    fix.arg=fix.arg, obs=data, ddistnam=ddistname, hessian=!is.null(gradient), method=meth, 
+                    ...), silent=TRUE)
+            }
+            else #cens == TRUE
+              opttryerror <- try(opt <- constrOptim(theta=vstart, f=fnobjcens, ui=Mat, ci=Bnd, grad=gradient,
+                    ddistnam=ddistname, rcens=rcens, lcens=lcens, icens=icens, ncens=ncens, pdistnam=pdistname,
+                    fix.arg=fix.arg, obs=data, hessian=!is.null(gradient), method=meth, 
+                    ...), silent=TRUE)
+            if(!inherits(opttryerror, "try-error"))
+              if(length(opt$counts) == 1) #appears when the initial point is a solution
+                opt$counts <- c(opt$counts, NA)
+            
+            
+          }else #opt.fun == "optim"
+          {
+            if(!cens)
+              opttryerror <- try(opt <- optim(par=vstart, fn=fnobj, fix.arg=fix.arg, obs=data, gr=gradient,
+                                              ddistnam=ddistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
+                                              ...), silent=TRUE)       
+            else #cens == TRUE
+              opttryerror <- try(opt <- optim(par=vstart, fn=fnobjcens, fix.arg=fix.arg, gr=gradient,
+                                              rcens=rcens, lcens=lcens, icens=icens, ncens=ncens, ddistnam=ddistname, 
+                                              pdistnam=pdistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
+                                              ...), silent=TRUE)   
+          }
+          
+        }else #hasbound == FALSE
+        {
+          opt.fun <- "optim"
+          if(!cens)
+            opttryerror <- try(opt <- optim(par=vstart, fn=fnobj, fix.arg=fix.arg, obs=data, gr=gradient,
+                                            ddistnam=ddistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
+                                            ...), silent=TRUE)       
+          else #cens == TRUE
+            opttryerror <- try(opt <- optim(par=vstart, fn=fnobjcens, fix.arg=fix.arg, gr=gradient,
+                                            rcens=rcens, lcens=lcens, icens=icens, ncens=ncens, ddistnam=ddistname, 
+                                            pdistnam=pdistname, hessian=TRUE, method=meth, lower=lower, upper=upper, 
+                                            ...), silent=TRUE) 
+        }
         options(warn=owarn)
         
         if (inherits(opttryerror, "try-error"))
@@ -208,8 +303,8 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
             warnings("The function optim encountered an error and stopped.")
             if(getOption("show.error.messages")) print(attr(opttryerror, "condition"))          
             return(list(estimate = rep(NA, length(vstart)), convergence = 100, loglik = NA, 
-                        hessian = NA, optim.function="optim", fix.arg = fix.arg, 
-                        optim.method=meth, fix.arg.fun = fix.arg.fun))
+                        hessian = NA, optim.function=opt.fun, fix.arg = fix.arg, 
+                        optim.method=meth, fix.arg.fun = fix.arg.fun, counts=c(NA, NA)))
         }
         
         if (opt$convergence>0) {
@@ -219,8 +314,9 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
         if(is.null(names(opt$par)))
           names(opt$par) <- names(vstart)
         res <- list(estimate = opt$par, convergence = opt$convergence, loglik = -opt$value, 
-                    hessian = opt$hessian, optim.function="optim", fix.arg = fix.arg, 
-                    optim.method=meth, fix.arg.fun = fix.arg.fun, weights = weights)
+                    hessian = opt$hessian, optim.function=opt.fun, fix.arg = fix.arg, 
+                    optim.method=meth, fix.arg.fun = fix.arg.fun, weights = weights, 
+                    counts=opt$counts, optim.message=opt$message)
     }
     else # Try to minimize the minus (log-)likelihood using a user-supplied optim function 
     {
@@ -239,7 +335,8 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
             warnings("The customized optimization function encountered an error and stopped.")
             if(getOption("show.error.messages")) print(attr(opttryerror, "condition"))          
             return(list(estimate = rep(NA, length(vstart)), convergence = 100, loglik = NA, 
-                        hessian = NA, optim.function=custom.optim, fix.arg = fix.arg, fix.arg.fun = fix.arg.fun))
+                        hessian = NA, optim.function=custom.optim, fix.arg = fix.arg, 
+                        fix.arg.fun = fix.arg.fun, counts=c(NA, NA)))
         }
         
         if (opt$convergence>0) {
@@ -256,7 +353,8 @@ mledist <- function (data, distr, start=NULL, fix.arg=NULL, optim.method="defaul
           names(opt$par) <- names(vstart)
         res <- list(estimate = opt$par, convergence = opt$convergence, loglik = -opt$value, 
                       hessian = opt$hessian, optim.function = custom.optim, fix.arg = fix.arg,
-                      method = method.cust, fix.arg.fun = fix.arg.fun, weights = weights)        
+                      method = method.cust, fix.arg.fun = fix.arg.fun, weights = weights, 
+                      counts=opt$counts, optim.message=opt$message)        
     }   
         
     return(res) 
